import logging
import mock
import yaml
import test_mock
import unittest
from os.path import join, dirname
from unittest import TestCase
from reviewrot.githubstack import GithubService
from reviewrot.gitlabstack import GitlabService
from reviewrot.pagurestack import PagureService
from reviewrot import get_git_service
from github.GithubException import BadCredentialsException
from gitlab.exceptions import GitlabConnectionError

# Disable logging to avoid messing up test output
logging.disable(logging.CRITICAL)


class GithubTest(TestCase):
    def setUp(self):
        filename = join(dirname(__file__), 'test_githubtest.yaml')
        with open(filename, 'r') as f:
            self.config = yaml.load(f)

    def test_object_create(self):
        self.assertTrue(isinstance((get_git_service('github')), GithubService))

    def test_request_review_token(self):
        github = GithubService()
        with self.assertRaises(BadCredentialsException) as context:
            github.request_reviews(user_name=self.config['user_name'],
                                   token=self.config['token'])
        self.assertTrue('Bad credentials' in str(context.exception))

    @mock.patch('github.Github.get_user', side_effect=test_mock.mock_get_user)
    def test_request_reviews_get_user(self, mock_get_user):
        with self.assertRaises(Exception) as context:
            GithubService().request_reviews(user_name=self.config['user_name'],
                                            token=self.config['token'])
        msg = 'Invalid username/organizaton: %s' % self.config['user_name']
        self.assertTrue(msg in str(context.exception))

    @mock.patch('github.Github.get_user', side_effect=test_mock.mock_get_user_)
    @mock.patch('reviewrot.githubstack.GithubService.get_reviews',
                side_effect=test_mock.mock_github_get_reviews)
    def test_request_reviews_with_repo(self, mock_get_user_,
                                       mock_github_get_reviews):
        res = GithubService().request_reviews(user_name=self.config['user_name'],
                                              token=self.config['token'],
                                              repo_name=self.config['repo_name'])
        self.assertEqual([self.config['msg']], res)

    @mock.patch('github.Github.get_user', side_effect=test_mock.mock_get_user_)
    @mock.patch('github.NamedUser.NamedUser.get_repos', side_effect=test_mock.mock_get_repos)
    @mock.patch('reviewrot.githubstack.GithubService.get_reviews',
                side_effect=test_mock.mock_github_get_reviews)
    def test_request_reviews_without_repo(self, mock_get_user_,
                                          mock_github_get_reviews, mock_get_repos):
        res = GithubService().request_reviews(user_name=self.config['user_name'],
                                              token=self.config['token'])
        self.assertEqual([self.config['msg']], res)

    @mock.patch('github.NamedUser.NamedUser.get_repo', side_effect=test_mock.mock_get_repo)
    def test_get_reviews_get_repo_not_found(self, mock_get_repo):
        with self.assertRaises(Exception) as context:
            uname = test_mock.mock_get_user_(self.config['user_name'])
            GithubService().get_reviews(uname=uname,
                                        repo_name=self.config['repo_name'])
        msg = 'Repository %s not found for user %s' % (self.config['repo_name'],
                                                       uname.login)
        self.assertTrue(msg in str(context.exception))

    @mock.patch('github.NamedUser.NamedUser.get_repo', side_effect=test_mock.mock_get_repo_)
    @mock.patch('github.Repository.Repository.get_pulls', side_effect=test_mock.mock_get_pulls)
    def test_get_reviews_get_repo(self, mock_get_pulls, mock_get_repo_):
        uname = test_mock.mock_get_user_(self.config['user_name'])
        res = GithubService().get_reviews(uname=uname,
                                          repo_name=self.config['repo_name'])
        self.assertEqual(res, [])


class GitlabTest(TestCase):
    def setUp(self):
        filename = join(dirname(__file__), 'test_gitlabtest.yaml')
        with open(filename, 'r') as f:
            self.config = yaml.load(f)

    def test_github_object_create(self):
        self.assertTrue(isinstance((get_git_service('gitlab')), GitlabService))

    def test_request_reviews__token(self):
        github = GitlabService()
        with self.assertRaises(GitlabConnectionError)as context:
            github.request_reviews(user_name=self.config['user_name'],
                                   token=self.config['token'],
                                   host=self.config['host'])
        self.assertTrue('NewConnectionError' in str(context.exception))

    @mock.patch('gitlab.Project.get', side_effect=test_mock.mock_projects_get)
    @mock.patch('gitlab.Gitlab.auth', side_effect=test_mock.mock_auth)
    def test_request_reviews_projects_get(self, mock_projects_get, mock_auth):
        with self.assertRaises(Exception) as context:
            GitlabService().request_reviews(user_name=self.config['user_name'],
                                            repo_name=self.config['repo_name'],
                                            token=self.config['token'],
                                            host=self.config['host'])
        msg = 'Project %s not found for user %s' % (self.config['repo_name'],
                                                    self.config['user_name'])
        self.assertTrue(msg in str(context.exception))

    @mock.patch('gitlab.GroupManager.search', side_effect=test_mock.mock_groups_search)
    @mock.patch('gitlab.Gitlab.auth', side_effect=test_mock.mock_auth)
    def test_request_reviews_groups_search(self, mock_groups_search, mock_auth):
        with self.assertRaises(Exception) as context:
            GitlabService().request_reviews(user_name=self.config['user_name'],
                                            token=self.config['token'],
                                            host=self.config['host'])
        msg = 'Invalid user/group name: %s' % self.config['user_name']
        self.assertTrue(msg in str(context.exception))

    @mock.patch('gitlab.Project.get', side_effect=test_mock.mock_projects_get_)
    @mock.patch('gitlab.Gitlab.auth', side_effect=test_mock.mock_auth)
    @mock.patch('reviewrot.gitlabstack.GitlabService.get_reviews',
                side_effect=test_mock.mock_gitlab_get_reviews)
    def test_request_reviews_with_repo(self, mock_projects_get_,
                                       mock_gitlab_get_reviews, mock_auth):
        res = GitlabService().request_reviews(user_name=self.config['user_name'],
                                              repo_name=self.config['repo_name'],
                                              token=self.config['token'],
                                              host=self.config['host'])
        self.assertEqual([self.config['msg']], res)


class PagureTest(TestCase):
    def setUp(self):
        filename = join(dirname(__file__), 'test_paguretest.yaml')
        with open(filename, 'r') as f:
            self.config = yaml.load(f)

    def test_pagure_object_create(self):
        self.assertTrue(isinstance((get_git_service('pagure')), PagureService))

    def test_request_review_incorrect_project_with_repo(self):
        pagure = PagureService()
        with self.assertRaises(Exception)as context:
            pagure.request_reviews(user_name=self.config['user_name'],
                                   repo_name=self.config['repo_name'])
        self.assertTrue('Project not found' in str(context.exception))


if __name__ == '__main__':
    unittest.main()



import os
import logging
import datetime
import requests
from reviewrot.basereview import BaseService, BaseReview

log = logging.getLogger(__name__)


class PagureService(BaseService):
    def __init__(self):
        self.session = requests.session()
        self.instance = "https://pagure.io"
        self.header = None

    def request_reviews(self, user_name, repo_name=None, state_=None,
                        value=None, duration=None, host=None, token=None):
        """
        Fetches merge requests by making API calls for specified
        username(namespace) and repo(project) name.
        Formats the merge requests details and print it on console.

        Args:
            user_name (str): Pagure username or organization name
            repo_name (str): Pagure repository name for specified
                             username or organization
            state_ (str): The filter(state) for pull requests, e.g, older
                          or newer
            value (int): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year, month,
                            hour, minute) for requests to be older or
                            newer than
            token (str): Pagure token for authentication
                         (Commented for unauthenticated request)
            host (str): Pagure host name (This value is not yet supported.
                        The default behavior is to use public pagure instance)
        Returns:
            res_ (list): Returns list of pull requests for specified
                         namespace and/or repo name
        """
        # Authenticated pagure object can be uncommented for future use
        # self.header = {"Authorization": "token " + token}
        if repo_name is not None:
            namespace = user_name
            request_url = "{}/api/0/{}/{}/pull-requests".format(self.instance,
                                                                namespace,
                                                                repo_name)
            log.debug('Looking for pull requests for %s -> %s/%s',
                      'pagure.io', namespace, repo_name)

        else:
            # absence of namespace, directly query pull requests for repo
            repo_name = user_name
            request_url = "{}/api/0/{}/pull-requests".format(self.instance,
                                                             repo_name)
            log.debug('Looking for pull requests for %s -> %s',
                      'pagure.io',  repo_name)
        log.debug('Calling API with request_url: %s', request_url)
        response = self._call_api(url=request_url)
        res_ = []
        for res in response['requests']:
            # if namespace exists in response
            try:
                repo_reference = os.path.join(res['project']['namespace'],
                                              res['project']['name'])
            except:
                repo_reference = res['project']['name']
            # format pull request url
            url = os.path.join('https://pagure.io/', repo_reference,
                               'pull-request', str(res['id']))
            # fetch the date pull request was filed at
            created_date = datetime.datetime.fromtimestamp(
                int(res['date_created'])).strftime('%Y-%m-%d %H:%M:%S')
            # format the date pull request was filed at
            try:
                date = datetime.datetime.strptime(created_date,
                                                  '%Y-%m-%d %H:%M:%S.%f')
            except ValueError:
                date = datetime.datetime.strptime(created_date,
                                                  '%Y-%m-%d %H:%M:%S')
            """ check if review request is older/newer than specified time
            interval"""
            result = self.check_request_state(date,
                                              state_, value, duration)
            if result is False:
                log.debug("pull request '%s' is not %s than specified"
                          " time interval", res['title'], state_)
                continue
            # format the time interval pull request has been filed since
            time = self.format_duration(created_at=date)
            res = PagureReview(user=res['user']['name'],
                               title=res['title'],
                               url=url,
                               time=time,
                               comments=len(res['comments']))
            log.info(res)
            res_.append(res)
        return res_

    def _call_api(self, url, method='GET'):
        """
        Method used to call the API.
        It returns the raw JSON returned by the API or raises an exception
        if something goes wrong.

        Args:
            method (str): the URL to call, can be GET, POST, DELETE, UPDATE...
                          Defaults to GET

        Returns:
            raw JSON returned by API
        """
        req = self.session.request(
            method=method,
            url=url,
            headers=self.header,
        )
        output = None
        try:
            output = req.json()
        except Exception as err:
            log.exception('Error while decoding JSON: {0}'.format(err))
            raise
        if req.status_code != 200:
            if 'error_code' in output:
                log.debug(output['error'])
                raise Exception(output['error'])
        return output


class PagureReview(BaseReview):
    pass



from reviewrot.githubstack import GithubService
from reviewrot.gitlabstack import GitlabService
from reviewrot.pagurestack import PagureService


def get_git_service(git):
    """
    Returns git service as per requested.

    Args:
        git (str): String indicating git service requested.

    Returns:
        Returns desired git service
    """
    if git == "github":
        return GithubService()
    elif git == "gitlab":
        return GitlabService()
    elif git == "pagure":
        return PagureService()
    else:
        raise ValueError('requested git service %s is not valid' % (git))



import os
import logging
import gitlab
import datetime
from reviewrot.basereview import BaseService, BaseReview
from gitlab.exceptions import GitlabGetError

log = logging.getLogger(__name__)


class GitlabService(BaseService):
    """
    This class represents Gitlab. The reference can be found here:
     https://docs.gitlab.com/ee/api/
    """
    def request_reviews(self, user_name, repo_name=None, state_=None,
                        value=None, duration=None, token=None, host=None):
        """
        Creates a gitlab object.
        Requests merge requests for specified username and repo name.
        If repo name is not provided then requests merge requests
        for all repos for specified username/organization.

        Args:
            user_name (str): Gitlab namespace
            repo_name (str): Gitlab project name for specified
                          namespace
            state_ (str): The state for pull requests, e.g, older
                        or newer
            value (int): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year, month,
                            hour, minute) for requests to be older or
                            newer than
            token (str): Gitlab token for authentication
            host (str): Gitlab host name for authentication

        Returns:
            response (list): Returns list of list of pull requests for
                             specified user(group) name and projectname or all
                             projectname for given groupname
        """
        gl = gitlab.Gitlab(host, token)
        gl.auth()
        log.debug('Gitlab instance created: %s', gl)
        response = []
        # if Repository name is explicitely provided
        if repo_name is not None:
            try:
                # get project object for given repo_name(project name)
                project = gl.projects.get(os.path.join(user_name, repo_name))
            except GitlabGetError:
                log.exception('Project %s not found for user %s',
                              repo_name, user_name)
                raise Exception('Project %s not found for user %s'
                                % (repo_name, user_name))
            # get merge requests for specified username and project name
            res = self.get_reviews(uname=user_name, project=project,
                                   state_=state_, value=value,
                                   duration=duration)
            # append incase of a non empty result
            if res:
                response.append(res)

        else:
            # get user object
            groups = gl.groups.search(user_name)
            if not groups:
                log.debug('Invalid user/group name: %s', user_name)
                raise Exception('Invalid user/group name: %s' % user_name)

            # get merge requests for all projects for specified group
            for group in groups:
                projects = gl.group_projects.list(group_id=group.id)
                if not projects:
                    log.debug("No projects found for user/group name %s",
                              user_name)
                for project in projects:
                    res = self.get_reviews(uname=user_name, project=project,
                                           state_=state_, value=value,
                                           duration=duration)
                # append incase of a non empty result
                if res:
                    response.append(res)
        return response

    def get_reviews(self, uname, project, state_=None,
                    value=None, duration=None):
        """
        Fetches merge requests for specified username(groupname)
        and repo(project) name.
        Formats the merge requests details and print it on console.

        Args:
            user_name (str): Gitlab namespace
            repo_name (str): Gitlab project name for specified
                             namespace
            state_ (str): The state for pull requests, e.g, older
                        or newer
            value (str): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year, month,
                            hour, minute) for requests to be older or
                            newer than.

        Returns:
            res_ (list): Returns list of pull requests for specified
                         user(group) name and project name
        """
        log.debug('Looking for merge requests for %s -> %s',
                  uname, project.name)

        # get list of open merge requests for a given repository(project)
        merge_requests = project.mergerequests.list(project_id=project.id,
                                                    state='opened')
        if not merge_requests:
            log.debug('No open merge requests found for %s/%s ',
                      uname, project.name)
        res_ = []
        for mr in merge_requests:
            try:
                mr_date = datetime.datetime.strptime(
                    mr.created_at, '%Y-%m-%dT%H:%M:%S.%fZ')
            except ValueError:
                mr_date = datetime.datetime.strptime(
                    mr.created_at, '%Y-%m-%dT%H:%M:%SZ')
            """ check if review request is older/newer than specified time
            interval"""
            result = self.check_request_state(mr_date,
                                              state_, value, duration)
            if result is False:
                log.debug("merge request '%s' is not %s than specified"
                          " time interval", mr.title, state_)
                continue
            # format the time interval pull request has been filed since
            time = self.format_duration(created_at=mr_date)
            res = GitlabReview(user=mr.author.username,
                               title=mr.title,
                               url=mr.web_url,
                               time=time,
                               comments=mr.user_notes_count)
            log.info(res)
            res_.append(res)
        return res_


class GitlabReview(BaseReview):
    pass



from os.path import join, dirname
import yaml
import github
import gitlab
from github.GithubException import UnknownObjectException
from gitlab.exceptions import GitlabGetError


with open(join(dirname(__file__), 'test_githubtest.yaml'), 'r') as f:
    github_config = yaml.load(f)

with open(join(dirname(__file__), 'test_gitlabtest.yaml'), 'r') as f:
    gitlab_config = yaml.load(f)

# github


def mock_get_user(user_name):
    raise UnknownObjectException('param1', 'param2')


def mock_get_user_(user_name):
    res = github.NamedUser.NamedUser('param1', 'param2',
                                     {'login': user_name}, 'param3')
    return res


def mock_get_repos():
    repo = github.Repository.Repository('param1', 'param2', 'param3', 'param4')
    res = []
    res.append(repo)
    return res


def mock_github_get_reviews(uname, repo_name, state_=None,
                            value=None, duration=None):
    msg = github_config['msg']
    return msg


def mock_get_repo(repo_name):
    raise UnknownObjectException('args', 'kwargs')


def mock_get_repo_(repo_name):
    repo = github.Repository.Repository('param1', 'param2',
                                        'param3', 'param4')
    return repo


def mock_get_pulls():
    return []

# gitlab


def mock_projects_get(user_name, repo_name):
    raise GitlabGetError()


def mock_projects_get_(user_name, repo_name):
    return gitlab.Project({'key': 'value'})


def mock_auth():
    return True


def mock_groups_search(user_name):
    return []


def mock_gitlab_get_reviews(uname, project, state_=None,
                            value=None, duration=None):
    msg = gitlab_config['msg']
    return msg



import logging
from reviewrot.basereview import BaseService, BaseReview
from github import Github
from github.GithubException import UnknownObjectException

log = logging.getLogger(__name__)


class GithubService(BaseService):
    """
    This class represents Github. The reference can be found here:
    https://developer.github.com/v3/
    """
    def request_reviews(self, user_name, repo_name=None, state_=None,
                        value=None, duration=None, token=None, host=None):
        """
        Creates a github object.
        Requests pull requests for specified username and repo name.
        If repo name is not provided then requests pull requests
        for all repos for specified username/organization.

        Args:
            user_name (str): Github username or organization name
            repo_name (str): Github repository name for specified
                             username or organization
            state_ (str): The filter state for pull requests, e.g, older
                          or newer
            value (int): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year, month, hour,
                            minute) for requests to be older or newer than
            token (str): Github token for authentication
            host (str): Github host name (This value is not yet supported.
                        Default behavior is to use public github instance.)
        Returns:
            response (list): Returns list of list of pull requests for
                             specified username and reponame or all reponame
                             for given username
        """
        # get authenticated github object
        g = Github(token)
        log.debug('Github instance created: %s', g)
        try:
            # get user object
            uname = g.get_user(user_name)
        except UnknownObjectException:
            log.exception('Invalid username/organizaton: %s', user_name)
            raise Exception('Invalid username/organizaton: %s' % user_name)
        response = []
        # if Repository name is explicitely provided
        if repo_name is not None:
            # get pull requests for specified username and repo name
            res = self.get_reviews(uname=uname, repo_name=repo_name,
                                   state_=state_, value=value,
                                   duration=duration)
            # append incase of a non empty result
            if res:
                response.append(res)
        else:
            # get all of the respositories for specified user/organization
            repo_list = uname.get_repos()
            if not repo_list:
                log.debug("No repositories found for user name %s", user_name)
            """
            list pull requests for all of the repositories for specified
            user/organization
            """
            for repo in repo_list:
                res = self.get_reviews(uname=uname, repo_name=repo.name,
                                       state_=state_, value=value,
                                       duration=duration)
                # append incase of a non empty result
                if res:
                    response.append(res)
        return response

    def get_reviews(self, uname, repo_name, state_=None,
                    value=None, duration=None):
        """
        Fetches pull requests for specified username and repo name.
        Formats the pull requests details and print it on console.

        Args:
            user_name (str): Github username or organization name
            repo_name (str): Github repository name for specified
                             username or organization
            state_ (str): The filter(state) for pull requests, e.g, older
                        or newer
            value (int): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year,
                            month, hour, minute) for requests to be
                            older or newer than
        Returns:
            res_ (list): Returns list of pull requests for specified
                         username and repo name
        """
        try:
            # get repository object for given user/organization and repo name
            repo = uname.get_repo(repo_name)
        except UnknownObjectException:
            log.exception('Repository %s not found for user %s',
                          repo_name, uname.login)
            raise Exception('Repository %s not found for user %s'
                            % (repo_name, uname.login))
        log.debug('Looking for pull requests for %s -> %s/%s ',
                  'github', uname.login, repo_name)
        # get list of open pull requests for a given repository
        pull_requests = repo.get_pulls()
        if not pull_requests:
            log.debug('No open pull requests found for %s/%s ',
                      uname.login, repo_name)
        res_ = []
        for pr in pull_requests:
            """ check if review request is older/newer than specified time
            interval"""
            result = self.check_request_state(pr.created_at,
                                              state_, value, duration)

            if result is False:
                # skip the current pull request
                log.debug("review request '%s' is not %s than specified"
                          " time interval", pr.title, state_)
                continue
            # format the time interval pull request has been filed since
            time = self.format_duration(created_at=pr.created_at)
            res = GithubReview(user=pr.user.login,
                               title=pr.title,
                               url=pr.html_url,
                               time=time,
                               comments=pr.comments)
            log.info(res)
            res_.append(res)
        return res_


class GithubReview(BaseReview):
    pass



from setuptools import setup, find_packages

with open("README.md", 'r') as f:
    long_description = f.read()

setup(name='review-rot',
      version='0.0',
      author='Nirzari Iyer',
      author_email='niyer@redhat.com',
      description=('CLI tool to list review(pull) requests from '
                   'github,gitlab,pagure and gerrit'),
      long_description=long_description,
      license='GNU GENERAL PUBLIC LICENSE Version 3',
      url='https://github.com/nirzari/review-rot',
      packages=find_packages(),
      install_requires=[
          'PyGithub',
          'PyYAML',
          'dateutils',
          'python-gitlab'
      ],
      tests_require=[
          'nose',
          'mock',
      ],
      test_suite='nose.collector',
      scripts=['bin/review-rot']
      )



from collections import OrderedDict
import datetime
from dateutil.relativedelta import relativedelta


class BaseService(object):

    def format_duration(self, created_at):
        """
        Formats the duration the review request is pending for
        Args:
            created_at (str): the date review request was filed

        Returns:
            a string of duration the review request is pending for
        """
        """
        find the relative time difference between now and
        review request filed to retrieve relative information
        """
        rel_diff = relativedelta(datetime.datetime.utcnow(),
                                 created_at)

        time_dict = OrderedDict([
            ('year', rel_diff.years),
            ('month', rel_diff.months),
            ('day', rel_diff.days),
            ('hour', rel_diff.hours),
            ('minute', rel_diff.minutes),
        ])

        result = []
        for k, v in time_dict.items():
            if v == 1:
                result.append('%s %s' % (v, k))
            elif v > 1:
                result.append('%s %ss' % (v, k))

        return ' '.join(result)

    def check_request_state(self, created_at,
                            state_, value, duration):
        """
        Checks if the review request is older or newer than specified
        time interval.
        Args:
            created_at (str): the date review request was filed
            state_ (str): state for review requests, e.g, older
                          or newer
            value (int): The value in terms of duration for requests
                         to be older or newer than
            duration (str): The duration in terms of period(year, month, hour
                            minute) for requests to be older or newer than

        Returns:
            True if the review request is older or newer than
            specified time interval, False otherwise
        """
        if (state_ is not None and value is not None and
                duration is not None):
            """
            find the relative time difference between now and
            review request filed to retrieve relative information
            """
            rel_diff = relativedelta(datetime.datetime.utcnow(),
                                     created_at)
            """
            find the absolute time difference between now and
            review request filed to retrieve absolute information
            """
            abs_diff = datetime.datetime.utcnow() - created_at

            if state_ not in ('older', 'newer'):
                raise ValueError('Invalid state value: %s' % state_)
            if duration == 'y':
                """ use rel_diff to retrieve absolute year since
                value of absolute and relative year is same."""
                if state_ == 'older' and rel_diff.years < value:
                    return False
                elif state_ == 'newer' and rel_diff.years >= value:
                    return False
            elif duration == 'm':
                """ use rel_diff to calculate absolute time difference
                in months """
                abs_month = (rel_diff.years*12) + rel_diff.months
                if state_ == 'older' and abs_month < value:
                    return False
                elif state_ == 'newer' and abs_month >= value:
                    return False
            elif duration == 'd':
                """ use abs_diff to retrieve absolute time difference
                in days """
                if state_ == 'older' and abs_diff.days < value:
                    return False
                elif state_ == 'newer' and abs_diff.days >= value:
                    return False
            elif duration == 'h':
                """ use abs_diff to calculate absolute time difference
                in hours """
                abs_hour = abs_diff.total_seconds()/3600
                if state_ == 'older' and abs_hour < value:
                    return False
                elif state_ == 'newer' and abs_hour >= value:
                    return False
            elif duration == 'min':
                """ use abs_diff to calculate absolute time difference
                in minutes """
                abs_min = abs_diff.total_seconds()/60
                if state_ == 'older' and abs_min < value:
                    return False
                elif state_ == 'newer' and abs_min >= value:
                    return False
            else:
                raise ValueError("Invalid duration type: %s" % duration)
        return True


class BaseReview(object):
    def __init__(self, user=None, title=None, url=None,
                 time=None, comments=None):
        self.user = user
        self.title = title
        self.url = url
        self.time = time
        self.comments = comments

    def __str__(self):
        if self.comments == 1:
            return ("@%s filed '%s' %s since %s with %s comment" % (
                self.user, self.title, self.url, self.time, self.comments))
        elif self.comments > 1:
            return ("@%s filed '%s' %s since %s with %s comments" % (
                self.user, self.title, self.url, self.time, self.comments))
        else:
            return ("@%s filed '%s' %s since %s" % (self.user, self.title,
                                                    self.url, self.time))
